# -*- coding:utf-8; eval: (blacken-mode) -*-
"""
Calculate anomalous current in bulk case
"""
import joblib
import numpy as np
from numpy import pi
import matplotlib.pyplot as plt
from scipy.linalg import expm
import logging
import contextlib

import usadelndsoc
from usadelndsoc.matsubara import get_matsubara_sum
from usadelndsoc.solver import *
from usadelndsoc.util import vectorize_parallel

import collections

mem = joblib.Memory("cache")


def get_solver(soc_alpha, eta, phi, L=10, h=0.5, D=1.0, n=5):
    sol = Solver(nx=n, ny=n)
    sol.mask[...] = MASK_NONE
    sol.Lx = L
    sol.Ly = L

    sol.Omega[...] = 0
    sol.Delta[0, :] = np.eye(2) * np.exp(1j * phi / 2)
    sol.Delta[-1, :] = np.eye(2) * np.exp(-1j * phi / 2)

    sol.mask[0, :] = MASK_TERMINAL
    sol.mask[-1, :] = MASK_TERMINAL

    sol.D = D
    sol.eta = eta

    nx, ny = sol.shape
    dx = sol.Lx / nx
    dy = sol.Ly / ny

    Ax = 2 * soc_alpha * np.kron(S_0, S_y)
    Ay = -2 * soc_alpha * np.kron(S_0, S_x)

    sol.Ux[...] = expm(1j * dx * Ax)
    sol.Uy[...] = expm(1j * dy * Ay)

    sol.Omega[1:-1] += h * np.kron(S_z, S_y)

    return sol


Res = collections.namedtuple("Res", ["x", "y", "J", "Jx", "Jy"])


@vectorize_parallel(returns_object=True, noarray=True)
@usadelndsoc.with_log_level(logging.WARNING)
def j(T, h, phi, n=15, eta=0.1, alpha_soc=0.1, L=10):

    sol = get_solver(soc_alpha=alpha_soc, eta=eta, L=L, n=n, phi=phi, h=h)

    E_typical = 10 + 10 * T
    w, a = get_matsubara_sum(T, E_typical)

    t3 = np.kron(S_z, S_0)
    res = sol.solve_many(omega=w)

    t3 = np.diag([1, 1, -1, -1])
    J = tr(res.J @ t3)
    J = -1j * pi * (J * a[:, None, None, None]).sum(axis=0)

    Jx = (J[:, :, 0] + J[:, :, 2]).real / 2
    Jy = (J[:, :, 1] + J[:, :, 3]).real / 2

    return (sol.x, sol.y, J, Jx, Jy)


def main():
    T = 0.1
    alpha_soc = 0.3
    h = np.r_[0.0, 0.25, 0.5, 1.0, 1.5]
    phi = np.linspace(-pi, pi, 37)

    res = j(T, h[:, None], phi[None, :], alpha_soc=alpha_soc, L=3, mem=mem)

    Jx_mean = np.asarray(
        [Res(*x).Jx[1:-1].sum(axis=1).mean(axis=0) for x in res.flat]
    ).reshape(res.shape)

    plt.plot(phi / pi, Jx_mean.T)
    plt.xlabel(r"$\varphi / \pi$")
    plt.ylabel(r"$I$")
    plt.legend()
    plt.savefig("cpr_sns.pdf")


if __name__ == "__main__":
    main()
